import requests
import time
import json

# Written by Christian Steinmetz.
# Version 1.0, 16.04.2020

def gaalopscriptFile2Python(filename):
    r"""
    Compiles a GAALOPScript file with GAALOPWeb and creates a Python file.
    The created Python file has the same name as the GAALOPScript file,
    but with replaced filename ending to .py.
    Convenience function, using function gaalopscriptString2Python().
        :param filename: The filename of the GAALOPScript file, that should be compiled with GAALOPWeb
        :type filename: str
        :returns: Was the compilation with GAALOPWeb successful?
        :rtype: boolean

    Written by Christian Steinmetz.    
    """
    # get scriptname from filename
    scriptName = filename
    scriptName = scriptName.replace(".clu", "")
    return gaalopscriptFile2PythonWithScriptName(filename, scriptName)

def gaalopscriptFile2PythonWithScriptName(filename, scriptName):
    r"""
    Compiles a GAALOPScript file with GAALOPWeb and creates a Python file.
    The name of the created Python file will be {scriptName}.py.
    Convenience function, using function gaalopscriptString2Python().
        :param filename: The filename of the GAALOPScript file, that should be compiled with GAALOPWeb
        :type filename: str
        :param scriptName: The name of the Python file, which should be generated, but without file ending.
        :type scriptName: str
        :returns: Was the compilation with GAALOPWeb successful?
        :rtype: boolean

    Written by Christian Steinmetz.    
    """
    with open(filename, 'rb') as f:
        return gaalopscriptString2Python(f.read(), scriptName)

def gaalopscriptString2Python(script, scriptName):
    r"""
    Compiles a GAALOPScript string with GAALOPWeb and creates a Python file.
    The name of the created Python file will be {scriptName}.py.
        :param script: The GAALOPScript, that should be compiled with GAALOPWeb
        :type script: str
        :param scriptName: The name of the Python file, which should be generated, but without file ending.
        :type scriptName: str
        :returns: Was the compilation with GAALOPWeb successful?
        :rtype: boolean

    Written by Christian Steinmetz.    
    """
    baseUrl = "https://gaalopweb.esa.informatik.tu-darmstadt.de/gaalopweb/res/"
    
    print("GAALOPScript will be uploaded now! Please wait.")

    r = requests.post(url = baseUrl+"api/compile", data=script, params= {"InputScript": scriptName+".clu",  "CodeGenerator": "de.gaalop.python.Plugin" }) 
    
    id = json.loads(r.content)["id"]

    print("Compilation started using the ID {id}! Please wait for the result.".format(id=id))

    # Wait on result compilationSucceeded or compilationException and check this each second
    progress = True
    while progress:
        time.sleep(1)

        # Get current status of compilation
        r = requests.get(url = baseUrl+"api/result/{id}".format(id=id))
        status = json.loads(r.content)["status"]

        # if compilation has ended successful, end the loop
        if status == "compilationSucceeded":
            progress = False

        # if compilation has ended with errors, abort the function
        if status == "compilationException":
            print("Compilation was NOT successful. You can find an error description on {baseUrl}cpp/result{id}".format(baseUrl=baseUrl, id=id))
            return False

    print("Compilation was successful. You will find the generated code in the file: {destinationFilename}".format(destinationFilename=scriptName+".py"))

    # Write result to destination file

    r = requests.get(url = baseUrl+"api/result/{id}/0".format(id=id))

    # Write result to python file
    with open(scriptName+".py", "w") as f:
        f.write(r.text)

    return True



